
%-----------------------------------------------------------------------
% THIS MAIN MATLAB CODE TESTS ON SEVERAL BIVARIATE DOMAINS SPECIFIED IN THE
% m-FILE define_domain_chebfun.m THE CUBATURE OF FUNCTIONS DEFINED IN THE
% m-FILE fct2D.m VIA GREEN FORMULA AND LEGENDRE TENSORIAL RULE ON THE
% SQUARE. MODIFYING THE VARIABLE cubature_type SEVERAL RULES THAT PROVIDES
% A CUBATURE RULE ON A RECTANGLE CAN BE USED FOR COMPUTING CUBATURE ON THE
% CURVILINEAR DOMAIN.
%-----------------------------------------------------------------------
% REFERENCE PAPERS.
%-------------------
%
% 1. A. SOMMARIVA and M. VIANELLO
%    "Gauss-like and triangulation-free cubature over polygons".
%
% 2. A. SOMMARIVA and M. VIANELLO
%    "Gauss-Green cubature over spline curvilinear polygons"
%
%-----------------------------------------------------------------------
% ADDITIONAL ROUTINES :
%----------------------
%
% 1. define_domain_chebfun
% 2. splinegauss_chebfun_2010
% 3. fct2D
% 4. reference_results
%
% THESE ROUTINES ARE EXTERNAL TO THIS DEMO FILE.
%-----------------------------------------------------------------------
%% Copyright (C) 2008 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: Nov. 2, 2009 - March. 30, 2011.
%-----------------------------------------------------------------------

clear all;
diary('numerical_experiments_spline_chebfun_gauss.m');
% clf;

%-----------------------------------------------------------------------
% domain_type :
%----------------------
%
% The domain is defined in "define_domain_chebfun.m". You can add Your own
% domain by adding in "define_domain_chebfun.m" your own domain via its
% boundary in parametric form.
%
% The value of "domain_type" correspond to the following choices:
%
% 1. Disk.
%   f=chebfun('cos(t)',[0 2*pi]); g=chebfun('sin(t)',[0 2*pi]);
%
% 2. Lune.
%    f=chebfun('0.5*(cos(t)+1)','0.5*cos((3*pi/2)-t)',[-pi/2 pi 3*pi/2]);
%    g=chebfun('0.5*(sin(t)+1)','0.5*sin((3*pi/2)-t)',[-pi/2 pi 3*pi/2]);
%
% 3. Union of two disks.
%    f=chebfun('cos(t)+sqrt(2)/2','cos(t-pi/2)-sqrt(2)/2',...
%            [-3*pi/4 3*pi/4 9/4*pi]);
%    g=chebfun('sin(t)','sin(t-pi/2)', [-3*pi/4 3*pi/4 9/4*pi]);
%
% 4. Intersection of two disks.
%    f=chebfun('cos(t)-sqrt(2)/2+1/2','cos(t+pi/2)+1/2+sqrt(2)/2',...
%            [-pi/4 pi/4 3/4*pi]);
%    g=chebfun('1/2+sin(t)','1/2+sin(t+pi/2)', [-pi/4 pi/4 3/4*pi]);
%
% 5. Cardioid.
%    f=chebfun('cos(t).*(1-cos(t))+1',[0 2*pi]);
%    g=chebfun('sin(t).*(1-cos(t))',[0 2*pi]);
%
% 6. Deltoid.
%    f=chebfun('2*1*cos(t)+1*cos(2*t)',[0 2*pi]); % 2*b*cos(t)+b*cos(2*t);
%    g=chebfun('2*1*sin(t)-1*sin(2*t)',[0 2*pi]); % 2*b*sin(t)-b*sin(2*t).
%
%-----------------------------------------------------------------------
domain_type_vett=2;

%-----------------------------------------------------------------------
% function_type :
%----------------------
%
% Function of which the user is interested in computing the integral in the
% domain defined by "domain_type". The functions are defined in "fct2D.m".
%
%    case 1
%        z=(x+y).^19;
%
%    case 2
%        z=exp(-(x-0.5).^2+(y-0.5).^2);
%
%    case 3
%        z=exp(-100*(x-0.5).^2+(y-0.5).^2);
%
%    case 4
%        z=( (x-0.5).^2 +(y-0.5).^2 ).^(1/2);
%
%    case 5
%        z=cos(20*(x+y));
%
%    case 6
%        z=franke(x,y);
%
%-----------------------------------------------------------------------
function_type=1;

%-----------------------------------------------------------------------
% Nvett :
%-----
%
% Degree of precision of the algebraic cubature rule. It is a vector,
% allowing multiple tests, for different degrees.
%-----------------------------------------------------------------------
Nvett=[11];
% Nvett=11;

%-----------------------------------------------------------------------
% cubature_type :
%----------------------
%
% The cubature is based on rules over a rectangle. We used the following
% examples.
%
% 0: Padua points.
% 1: Fejer 1 type (tensorial).
% 2: Fejer 2 type (tensorial).
% 3: Clenshaw-Curtis (tensorial).
% 4: Gauss-Legendre (tensorial).
% 5: Gauss-Legendre-Lobatto (tensorial).
%
% Use as default "cubature_type = 4" that correspond to a Gauss-Legendre
% tensorial formula (as in the paper). The other cubature formulae are
% given just for sake of completeness.
%-----------------------------------------------------------------------

cubature_type=4;

%-----------------------------------------------------------------------
% rotation, P, Q :
%----------------------
%
% Choosing a baseline (segment joining P and Q), one can provide a cubature
% rule with all the points inside the domain.
%
% 0: no rotation.
% 1: rotation decided automatically. Good for convex polygons.
% 2. rotation provided by the user.
%-----------------------------------------------------------------------

rotation=2;

%-----------------------------------------------------------------------
% reference_cubature_type :
%----------------------
%
% If reference_result=[]  is left empty then a reference result
% will be computed by a high degree of precision cubature formula.
%
% 0: Padua points.
% 1: Fejer 1 type (tensorial).
% 2: Fejer 2 type (tensorial).
% 3: Clenshaw-Curtis (tensorial).
% 4: Gauss-Legendre (tensorial).
% 5: Gauss-Legendre-Lobatto (tensorial).
%
% Default: reference_cubature_type=4
%
%-----------------------------------------------------------------------

reference_cubature_type=4;

%-----------------------------------------------------------------------
% tol :
%----------------------
%
% Chebfun tolerance. Used to describe the boundary.
%-----------------------------------------------------------------------

tol=eps;
tol_ref=eps;

%-----------------------------------------------------------------------
% Nref :
%----------------------
%
% ChebfunGauss algebraic degree of precision for computing the reference
% result (if "reference_result=[]").
%-----------------------------------------------------------------------

Nref=min(max(2*max(Nvett),50),100);


% ------------ MAIN PROGRAM STARTS HERE -----------------------------------

for index_domain=1:length(domain_type_vett)

    domain_type=domain_type_vett(index_domain);

    switch domain_type
        case 1
            P=[0 0]; Q=[0 1]; %Disk.
        case 2
            P=[0.5 0]; Q=[0.5 1]; % Lune (good baseline).
            P=[0 0]; Q=[0 1]; % Lune (bad baseline). 
        case 3
            P=[-1 0]; Q=[1 0]; % Union of two disks.
        case 4
            P=[0.5 0]; Q=[0.5 1]; % Intersection of two disks.
        case 5
            P=[0.25 0]; Q=[0.25 1]; %Cardioid.
        case 6
            P=[-1.5 0]; Q=[3 0]; % Deltoid.
        otherwise
            P=[0 0]; Q=[0 1];
    end

    %-----------------------------------------------------------------------
    % reference_result :
    %----------------------
    %
    % Reference result. If this variable is left empty then a reference result
    % will be computed by a high degree of precision cubature formula.
    %
    %-----------------------------------------------------------------------

    % reference_result=[];
    reference_result=reference_results(domain_type, function_type);

    %-----------------------------------------------------------------------
    % REFERENCE CUBATURE RESULT.
    %-----------------------------------------------------------------------

    if length(reference_result) == 0

        % DEFINING DOMAIN.
        [f,g,f1,g1,table_ends_degrees_extended]=...
            define_domain_chebfun(domain_type,tol_ref);

        % table_ends_degrees_extended=[fends(1) fd(1) gd(1);
        %                                 table_ends_degrees]
        fends=table_ends_degrees_extended(:,1);
        table_ends_degrees=table_ends_degrees_extended(2:end,:);

        % CONSIDERING END POINTS TO MAKE "BLOCKS".
        polygon_sides=[feval(f,fends(1)) feval(g,fends(1)); ...
            feval(f,table_ends_degrees(:,1)) ...
            feval(g,table_ends_degrees(:,1))];

        fprintf('\n \t >>CHEBFUNGAUSS REFERENCE (DEG. %5.0f)', Nref);

        % SPLINEGAUSS NODES AND WEIGHTS.
        [nodes_x_ref,nodes_y_ref,weights_ref]=splinegauss_chebfun_2010(Nref,...
            f,g,f1,g1,polygon_sides,rotation,P,Q,...
            table_ends_degrees_extended,reference_cubature_type);

        % EVALUATING CUBATURE.
        f_xy_ref=fct2D(nodes_x_ref,nodes_y_ref,function_type);
        I_ref=f_xy_ref'*weights_ref;

    else

        fprintf('\n \t >>CHEBFUNGAUSS REFERENCE RESULT ASSIGNED BY USER');
        I_ref=reference_result;

    end

    rel_err=[]; I_hist=[];
    
    for index=1 : length(Nvett)

        N=Nvett(index);

        %-------------------------------------------------------------------
        % CHEBFUNGAUSS
        %-------------------------------------------------------------------
        % DEFINING DOMAIN.
        [f,g,f1,g1,table_ends_degrees_extended]=...
            define_domain_chebfun(domain_type,tol);

        % table_ends_degrees_extended=[fends(1) fd(1) gd(1); table_ends_degrees]
        % TABLE OF END POINTS, f,g DEGREES.
        fends=table_ends_degrees_extended(:,1);
        table_ends_degrees=table_ends_degrees_extended(2:end,:);

        % CONSIDERING END POINTS TO MAKE "BLOCKS".
        polygon_sides=[feval(f,fends(1)) feval(g,fends(1)); ...
            feval(f,table_ends_degrees(:,1)) feval(g,table_ends_degrees(:,1))];

        fprintf('\n \t >>CHEBFUNGAUSS (DEG. %5.0f)', N);

        cputimes(1)=cputime;

        % DETERMINING CUBATURE POINTS AND WEIGHTS.
        [nodes_x,nodes_y,weights]=splinegauss_chebfun_2010(N,f,g,f1,g1,...
            polygon_sides,rotation,P,Q,table_ends_degrees_extended,...
            cubature_type);

        % EVALUATING CUBATURE.
        f_xy=fct2D(nodes_x,nodes_y,function_type);
        I=f_xy'*weights;
        
        I_hist=[I_hist; I];

        cputimes(2)=cputime;
        cpuspg=cputimes(2)-cputimes(1);



        %-----------------------------------------------------------------------
        % STATS.
        %-----------------------------------------------------------------------
        fprintf('\n \n \t ----------------------------------------------------')
        fprintf('\n \t              CHEBFUN BOUNDARY PROPERTIES')
        fprintf('\n \t ----------------------------------------------------')

        fprintf('\n \n \t [BOUNDARY CHEBFUN TOL.]: %2.2e \n ', tol);


        for index=1:size(table_ends_degrees_extended,1)-1
            fprintf('\n \t [PT1]: %3.4f [PT3]: %3.4f [Deg(f)]: %3.0f [Deg(g)]: %3.0f',...
                table_ends_degrees_extended(index,1),...
                table_ends_degrees_extended(index+1,1),...
                table_ends_degrees_extended(index,2),...
                table_ends_degrees_extended(index,3));


        end


        fprintf('\n\n \t ----------------------------------------------------')
        fprintf('\n \t                     CHEBFUN GAUSS')
        fprintf('\n \t ----------------------------------------------------')

        fprintf('\n \n \t [CUBATURE]: %1.25e',I);

        if I_ref == 0
            rel_err_loc=abs(I-I_ref);
            fprintf('\n \n \t [SPG vs SPG REF. (DEG. %3.0f). ABS.ERR.]: %2.2e',...
                Nref,rel_err_loc);
        else
            rel_err_loc=abs(I-I_ref)/abs(I_ref);
            fprintf('\n \n \t [SPG vs SPG REF. (DEG. %3.0f). REL.ERR.]: %2.2e',...
                Nref,rel_err_loc);
        end
        fprintf('\n \t [ADE]: %5.0f [NODES]: %10.0f [FUNCTION]: %2.0f',N,...
            length(f_xy),function_type);
        
        rel_err=[rel_err; rel_err_loc];
        
        switch cubature_type

            % 0: Padua points.
            % 1: Fejer 1 type (tensorial).
            % 2: Fejer 2 type (tensorial).
            % 3: Clenshaw-Curtis (tensorial).
            % 4: Gauss-Legendre (tensorial).
            % 5: Gauss-Legendre-Lobatto (tensorial).

            case 0
                fprintf('\n \n \t [PADUA TYPE CUBATURE]');
            case 1
                fprintf('\n \n \t [FEJER TYPE 1 TENSORIAL CUBATURE]');
            case 2
                fprintf('\n \n \t [FEJER TYPE 2 TENSORIAL CUBATURE]');
            case 3
                fprintf('\n \n \t [CLENSHAW-CURTIS TENSORIAL CUBATURE]');
            case 4
                fprintf('\n \n \t [GAUSS-LEGENDRE TENSORIAL CUBATURE]');
            case 5
                fprintf('\n \n \t [GAUSS-LEGENDRE-LOBATTO TENSORIAL CUB.]');

        end

        switch domain_type

            % 0: Padua points.
            % 1: Fejer 1 type (tensorial).
            % 2: Fejer 2 type (tensorial).
            % 3: Clenshaw-Curtis (tensorial).
            % 4: Gauss-Legendre (tensorial).
            % 5: Gauss-Legendre-Lobatto (tensorial).

            case 1
                fprintf('\n \n \t [DOMAIN]: DISK \n');
            case 2
                fprintf('\n \n \t [DOMAIN]: CARDIOID \n');
            case 3
                fprintf('\n \n \t [DOMAIN]: LUNE \n');
            case 4
                fprintf('\n \n \t [DOMAIN]: INTERSECTION OF DISKS \n');
            case 5
                fprintf('\n \n \t [DOMAIN]: UNION OF DISKS \n');
            otherwise
                fprintf('\n \n \t [DOMAIN]: OTHER DOMAIN \n');
        end


        % TESTING NUMBER OF CUBATURE NODES IN THE GAUSSIAN CASE.
        if (cubature_type == 4)
            for index=1:size(table_ends_degrees,1)
                Ntau=floor(N/2)+1;
                ss=(2*Ntau-1)*max(table_ends_degrees(index,2),...
                    table_ends_degrees(index,3))+...
                    (table_ends_degrees(index,2)+table_ends_degrees(index,3))-1;
                Nu=floor(ss/2)+1;

                number_pts(index)=Ntau*Nu;
            end
            ss=sum(number_pts);

            fprintf('\n \t [THEORETICAL NUMBER OF NODES]: %10.0f',ss);
        end

        fprintf('\n\n \t ----------------------------------------------------')
        fprintf('\n \t                  SPLINEGAUSS REFERENCE ');
        fprintf('\n \t ----------------------------------------------------')

        fprintf('\n \n \t [CUBATURE]: %1.17e',I_ref);


        fprintf('\n\n \t ----------------------------------------------------')
        fprintf('\n \t                       CPUTIMES')
        fprintf('\n \t ----------------------------------------------------')
        fprintf('\n \n \t [SPG]: %2.2e',cpuspg);
        fprintf('\n \n \t =====================================================')
    end

    fprintf('\n\n \t ----------------------------------------------------')
    fprintf('\n \t                       SUMMARY')
    fprintf('\n \t ----------------------------------------------------')

    for sss=1:length(Nvett)
        fprintf('\n \t ADE: %3.0f R.ERR.: %1.1e RES: %1.15e',...
            Nvett(sss),rel_err(sss),I_hist(sss));
    end
    fprintf('\n \t ----------------------------------------------------')


    fprintf('\n \n  \n\t ++++++++++++++++++++++++++++++++++++++++++++++++++++')



end

% plot(nodes_x,nodes_y,'r+'); axis square;

diary off

fprintf('\n \n');