function Iref=reference_results(domain_type, function_type)

%-----------------------------------------------------------------------
% domain_type :
%----------------------
%
% The domain is defined in "define_domain_chebfun.m". You can add Your own
% domain by adding in "define_domain_chebfun.m" your own domain via its
% boundary in parametric form.
%
% The value of "domain_type" correspond to the following choices:
%
% 1. Disk.
%   f=chebfun('cos(t)',[0 2*pi]); g=chebfun('sin(t)',[0 2*pi]);
%
% 2. Cardioid.
%    f=chebfun('cos(t).*(1-cos(t))+1',[0 2*pi]);
%    g=chebfun('sin(t).*(1-cos(t))',[0 2*pi]);
%
% 3. Lune.
%    f=chebfun('0.5*(cos(t)+1)','0.5*cos((3*pi/2)-t)',[-pi/2 pi 3*pi/2]);
%    g=chebfun('0.5*(sin(t)+1)','0.5*sin((3*pi/2)-t)',[-pi/2 pi 3*pi/2]);
%
% 4. Union of two disks.
%    f=chebfun('cos(t)+sqrt(2)/2','cos(t-pi/2)-sqrt(2)/2',...
%            [-3*pi/4 3*pi/4 9/4*pi]);
%    g=chebfun('sin(t)','sin(t-pi/2)', [-3*pi/4 3*pi/4 9/4*pi]);
%
% 5. Intersection of two disks.
%    f=chebfun('cos(t)-sqrt(2)/2+1/2','cos(t+pi/2)+1/2+sqrt(2)/2',...
%            [-pi/4 pi/4 3/4*pi]);
%    g=chebfun('1/2+sin(t)','1/2+sin(t+pi/2)', [-pi/4 pi/4 3/4*pi]);
%
% 6. Deltoid.
%    f=chebfun('2*1*cos(t)+1*cos(2*t)',[0 2*pi]); % 2*b*cos(t)+b*cos(2*t);
%    g=chebfun('2*1*sin(t)-1*sin(2*t)',[0 2*pi]); % 2*b*sin(t)-b*sin(2*t).
%
%-----------------------------------------------------------------------
% function_type :
%----------------------
%
% Function of which the user is interested in computing the integral in the
% domain defined by "domain_type". The functions are defined in "fct2D.m".
%%-----------------------------------------------------------------------
% function_type :
%----------------------
%
% Function of which the user is interested in computing the integral in the
% domain defined by "domain_type". The functions are defined in "fct2D.m".
%
%    case 1
%        z=(x+y).^19;
%
%    case 2
%        z=exp(-(x-0.5).^2+(y-0.5).^2);
%
%    case 3
%        z=exp(-100*(x-0.5).^2+(y-0.5).^2);
%
%    case 4
%        z=( (x-0.5).^2 +(y-0.5).^2 ).^(1/2);
%
%    case 5
%        z=cos(20*(x+y));
%
%    case 6
%        z=franke(x,y).
%-----------------------------------------------------------------------

switch domain_type

    case 1 % DISK.

        switch function_type
            case 1
                % ESTIMATED RELATIVE ERROR: 0.
                Iref=0;
            case 2
                % ESTIMATED RELATIVE ERROR: 10^(-15).
                Iref=1.476139002266508;
            case 3
                % ESTIMATED RELATIVE ERROR: 10^(-14).
                Iref=3.141527827120767*(10^(-2));
            case 4
                % ESTIMATED RELATIVE ERROR: 10^(-8).
                Iref=2.854079917515110;
            case 5
                % ESTIMATED RELATIVE ERROR: 10^(-14)
                Iref=2.301872593651162*(10^(-2));
            case 6
                % ESTIMATED RELATIVE ERROR: 10^(-15).
                Iref=1.851989839087119;
            otherwise
                Iref=[];
        end

    case 2 % LUNE.

        switch function_type
            case 1
                % ESTIMATED RELATIVE ERROR: 4E-14.
                Iref=6.385574327469890*(10^(+2));
            case 2
                % ESTIMATED RELATIVE ERROR: 1E-15.
                Iref=5.726372043252941*(10^(-1));
            case 3
                % ESTIMATED RELATIVE ERROR: 2E-14.
                Iref=3.137185199245524*(10^(-2));
            case 4
                % ESTIMATED RELATIVE ERROR: 4E-11.
                Iref=2.064677029709676*(10^(-1));
            case 5
                % ESTIMATED RELATIVE ERROR: 5E-14.
                Iref=6.289581219565822*(10^(-3));
            case 6
                % ESTIMATED RELATIVE ERROR: 1E-14.
                Iref=2.030762698534236*(10^(-1));
            otherwise
                Iref=[];
        end

    case 3 % UNION OF TWO DISKS.

        switch function_type
            case 1
                % ESTIMATED RELATIVE ERROR: 7E-01.
              %  Iref=1.48710002014055437*(10^(-10));
              Iref=5.584293661167750e-11; % ADE 19.
            case 2
                % ESTIMATED RELATIVE ERROR: 6E-15.
                Iref=2.039766602159862*(10^(0));
            case 3
                % ESTIMATED RELATIVE ERROR: 9E-14.
                Iref=3.141592653403229*(10^(-2));
            case 4
                % ESTIMATED RELATIVE ERROR: 6E-08.
                Iref=6.528597310412948;
            case 5
                % ESTIMATED RELATIVE ERROR: 3E-13.
                Iref=-1.231537275802433*(10^(-2));
            case 6
                % ESTIMATED RELATIVE ERROR: 8E-16.
                Iref=2.065871572420205;
            otherwise
                Iref=[];
        end

    case 4 % INTERSECTION OF TWO DISKS.

        switch function_type
            case 1
                % ESTIMATED RELATIVE ERROR: 3E-14.
                Iref=4.570643824458973*(10^(+2));
            case 2
                % ESTIMATED RELATIVE ERROR: 6E-15.
                Iref=5.068098577369264*(10^(-1));
            case 3
                % ESTIMATED RELATIVE ERROR: 2E-14.
                Iref=3.141463000651704*(10^(-2));
            case 4
                % ESTIMATED RELATIVE ERROR: 4E-09.
                Iref=1.827876057321204*(10^(-1));
            case 5
                % ESTIMATED RELATIVE ERROR: 1E-13.
                Iref=4.932336162031037*(10^(-3));
            case 6
                % ESTIMATED RELATIVE ERROR: 6E-16.
                Iref=2.163090164260847*(10^(-1));
            otherwise
                Iref=[];
        end

    case 5 % CARDIOID.

        switch function_type
            case 1
                % ESTIMATED RELATIVE ERROR: 10^(-16).
                Iref=2.271851704296164*(10^4);
            case 2
                % ESTIMATED RELATIVE ERROR: 10^(-15).
                Iref=2.080016120389035;
            case 3
                % ESTIMATED RELATIVE ERROR: 10^(-14).
                Iref=3.141592653589625*(10^(-2));
            case 4
                % ESTIMATED RELATIVE ERROR: 10^(-9).
                Iref=4.547551380452429;
            case 5
                % ESTIMATED RELATIVE ERROR: 10^(-14)
                Iref=7.169671894735140*(10^(-3));
            case 6
                % ESTIMATED RELATIVE ERROR: 10^(-16).
                Iref=2.578576654611357;
            otherwise
                Iref=[];
        end

    case 6 % DELTOID.

        switch function_type
            case 1
                % ESTIMATED RELATIVE ERROR: 5E-14.
                Iref=-1.79870094744327593e+09;
            case 2
                % ESTIMATED RELATIVE ERROR: 9E-15.
                Iref=1.99622517119882525e+00;
            case 3
                % ESTIMATED RELATIVE ERROR: 3E-15.
                Iref=3.14158274078792238e-02;
            case 4
                % ESTIMATED RELATIVE ERROR: 5E-08.
                Iref=7.95496632405762583e+00;
            case 5
                % ESTIMATED RELATIVE ERROR: 9E-14.
                Iref=3.87582554439577648e-02;
            otherwise
                Iref=[];
        end

    otherwise
        Iref=[];

end



